@@ -1,6 +1,8 @@
 import {
   Activity,
   ApiKey,
+  AssetFace,
+  AssetFile,
   AuthApiKey,
   AuthSharedLink,
   AuthUser,
@@ -8,12 +10,16 @@ import {
   Library,
   Memory,
   Partner,
+  Person,
   Session,
+  Stack,
+  Tag,
   User,
   UserAdmin,
 } from 'src/database';
 import { MapAsset } from 'src/dtos/asset-response.dto';
 import { AuthDto } from 'src/dtos/auth.dto';
+import { AssetEditAction, AssetEditActionItem, MirrorAxis } from 'src/dtos/editing.dto';
 import { QueueStatisticsDto } from 'src/dtos/queue.dto';
 import {
   AssetFileType,
@@ -22,10 +28,11 @@ import {
   AssetVisibility,
   MemoryType,
   Permission,
+  SourceType,
   UserMetadataKey,
   UserStatus,
 } from 'src/enum';
-import { OnThisDayData, UserMetadataItem } from 'src/types';
+import { DeepPartial, OnThisDayData, UserMetadataItem } from 'src/types';
 import { v4, v7 } from 'uuid';
 
 export const newUuid = () => v4();
@@ -159,11 +166,18 @@ const queueStatisticsFactory = (dto?: Partial<QueueStatisticsDto>) => ({
   ...dto,
 });
 
-const stackFactory = () => ({
-  id: newUuid(),
-  ownerId: newUuid(),
-  primaryAssetId: newUuid(),
-});
+const stackFactory = ({ owner, assets, ...stack }: DeepPartial<Stack> = {}): Stack => {
+  const ownerId = newUuid();
+
+  return {
+    id: newUuid(),
+    primaryAssetId: assets?.[0].id ?? newUuid(),
+    ownerId,
+    owner: userFactory(owner ?? { id: ownerId }),
+    assets: assets?.map((asset) => assetFactory(asset)) ?? [],
+    ...stack,
+  };
+};
 
 const userFactory = (user: Partial<User> = {}) => ({
   id: newUuid(),
@@ -222,36 +236,43 @@ const userAdminFactory = (user: Partial<UserAdmin> = {}) => {
   };
 };
 
-const assetFactory = (asset: Partial<MapAsset> = {}) => ({
-  id: newUuid(),
-  createdAt: newDate(),
-  updatedAt: newDate(),
-  deletedAt: null,
-  updateId: newUuidV7(),
-  status: AssetStatus.Active,
-  checksum: newSha1(),
-  deviceAssetId: '',
-  deviceId: '',
-  duplicateId: null,
-  duration: null,
-  encodedVideoPath: null,
-  fileCreatedAt: newDate(),
-  fileModifiedAt: newDate(),
-  isExternal: false,
-  isFavorite: false,
-  isOffline: false,
-  libraryId: null,
-  livePhotoVideoId: null,
-  localDateTime: newDate(),
-  originalFileName: 'IMG_123.jpg',
-  originalPath: `/data/12/34/IMG_123.jpg`,
-  ownerId: newUuid(),
-  stackId: null,
-  thumbhash: null,
-  type: AssetType.Image,
-  visibility: AssetVisibility.Timeline,
-  ...asset,
-});
+const assetFactory = (
+  asset: Omit<DeepPartial<MapAsset>, 'exifInfo' | 'owner' | 'stack' | 'tags' | 'faces' | 'files' | 'edits'> = {},
+) => {
+  return {
+    id: newUuid(),
+    createdAt: newDate(),
+    updatedAt: newDate(),
+    deletedAt: null,
+    updateId: newUuidV7(),
+    status: AssetStatus.Active,
+    checksum: newSha1(),
+    deviceAssetId: '',
+    deviceId: '',
+    duplicateId: null,
+    duration: null,
+    encodedVideoPath: null,
+    fileCreatedAt: newDate(),
+    fileModifiedAt: newDate(),
+    isExternal: false,
+    isFavorite: false,
+    isOffline: false,
+    libraryId: null,
+    livePhotoVideoId: null,
+    localDateTime: newDate(),
+    originalFileName: 'IMG_123.jpg',
+    originalPath: `/data/12/34/IMG_123.jpg`,
+    ownerId: newUuid(),
+    stackId: null,
+    thumbhash: null,
+    type: AssetType.Image,
+    visibility: AssetVisibility.Timeline,
+    width: null,
+    height: null,
+    isEdited: false,
+    ...asset,
+  };
+};
 
 const activityFactory = (activity: Partial<Activity> = {}) => {
   const userId = activity.userId || newUuid();
@@ -331,6 +352,7 @@ const assetSidecarWriteFactory = () => {
         id: newUuid(),
         path: '/path/to/original-path.jpg.xmp',
         type: AssetFileType.Sidecar,
+        isEdited: false,
       },
     ],
     exifInfo: {
@@ -358,6 +380,7 @@ const assetOcrFactory = (
     boxScore?: number;
     textScore?: number;
     text?: string;
+    isVisible?: boolean;
   } = {},
 ) => ({
   id: newUuid(),
@@ -373,13 +396,120 @@ const assetOcrFactory = (
   boxScore: 0.95,
   textScore: 0.92,
   text: 'Sample Text',
+  isVisible: true,
   ...ocr,
 });
 
+const assetFileFactory = (file: Partial<AssetFile> = {}) => ({
+  id: newUuid(),
+  type: AssetFileType.Preview,
+  path: '/uploads/user-id/thumbs/path.jpg',
+  isEdited: false,
+  isProgressive: false,
+  ...file,
+});
+
+const exifFactory = (exif: Partial<Exif> = {}) => ({
+  assetId: newUuid(),
+  autoStackId: null,
+  bitsPerSample: null,
+  city: 'Austin',
+  colorspace: null,
+  country: 'United States of America',
+  dateTimeOriginal: newDate(),
+  description: '',
+  exifImageHeight: 420,
+  exifImageWidth: 42,
+  exposureTime: null,
+  fileSizeInByte: 69,
+  fNumber: 1.7,
+  focalLength: 4.38,
+  fps: null,
+  iso: 947,
+  latitude: 30.267_334_570_570_195,
+  longitude: -97.789_833_534_282_07,
+  lensModel: null,
+  livePhotoCID: null,
+  make: 'Google',
+  model: 'Pixel 7',
+  modifyDate: newDate(),
+  orientation: '1',
+  profileDescription: null,
+  projectionType: null,
+  rating: 4,
+  state: 'Texas',
+  tags: ['parent/child'],
+  timeZone: 'UTC-6',
+  ...exif,
+});
+
+const tagFactory = (tag: Partial<Tag>): Tag => ({
+  id: newUuid(),
+  color: null,
+  createdAt: newDate(),
+  parentId: null,
+  updatedAt: newDate(),
+  value: `tag-${newUuid()}`,
+  ...tag,
+});
+
+const faceFactory = ({ person, ...face }: DeepPartial<AssetFace> = {}): AssetFace => ({
+  assetId: newUuid(),
+  boundingBoxX1: 1,
+  boundingBoxX2: 2,
+  boundingBoxY1: 1,
+  boundingBoxY2: 2,
+  deletedAt: null,
+  id: newUuid(),
+  imageHeight: 420,
+  imageWidth: 42,
+  isVisible: true,
+  personId: null,
+  sourceType: SourceType.MachineLearning,
+  updatedAt: newDate(),
+  updateId: newUuidV7(),
+  person: person === null ? null : personFactory(person),
+  ...face,
+});
+
+const assetEditFactory = (edit?: Partial<AssetEditActionItem>): AssetEditActionItem => {
+  switch (edit?.action) {
+    case AssetEditAction.Crop: {
+      return { action: AssetEditAction.Crop, parameters: { height: 42, width: 42, x: 0, y: 10 }, ...edit };
+    }
+    case AssetEditAction.Mirror: {
+      return { action: AssetEditAction.Mirror, parameters: { axis: MirrorAxis.Horizontal }, ...edit };
+    }
+    case AssetEditAction.Rotate: {
+      return { action: AssetEditAction.Rotate, parameters: { angle: 90 }, ...edit };
+    }
+    default: {
+      return { action: AssetEditAction.Mirror, parameters: { axis: MirrorAxis.Vertical } };
+    }
+  }
+};
+
+const personFactory = (person?: Partial<Person>): Person => ({
+  birthDate: newDate(),
+  color: null,
+  createdAt: newDate(),
+  faceAssetId: null,
+  id: newUuid(),
+  isFavorite: false,
+  isHidden: false,
+  name: 'person',
+  ownerId: newUuid(),
+  thumbnailPath: '/path/to/person/thumbnail.jpg',
+  updatedAt: newDate(),
+  updateId: newUuidV7(),
+  ...person,
+});
+
 export const factory = {
   activity: activityFactory,
   apiKey: apiKeyFactory,
   asset: assetFactory,
+  assetFile: assetFileFactory,
   assetOcr: assetOcrFactory,
   auth: authFactory,
   authApiKey: authApiKeyFactory,
@@ -396,6 +526,11 @@ export const factory = {
   jobAssets: {
     sidecarWrite: assetSidecarWriteFactory,
   },
+  exif: exifFactory,
+  face: faceFactory,
+  person: personFactory,
+  assetEdit: assetEditFactory,
+  tag: tagFactory,
   uuid: newUuid,
   date: newDate,
   responses: {
